@@ -18,8 +18,6 @@
 # with yams. If not, see <http://www.gnu.org/licenses/>.
 """unit tests for module yams.schema classes"""
 
-from __future__ import with_statement
-
 from logilab.common.testlib import TestCase, unittest_main
 
 from copy import copy, deepcopy
@@ -30,13 +28,15 @@ from six import text_type
 from yams import (BASE_TYPES, ValidationError, BadSchemaDefinition,
                   register_base_type, unregister_base_type)
 from yams.buildobjs import (register_base_types, make_type, _add_relation,
-                            EntityType, RelationType, RelationDefinition)
+                            EntityType, RelationType, RelationDefinition,
+                            RichString)
 from yams.schema import Schema, RelationDefinitionSchema
 from yams.interfaces import IVocabularyConstraint
 from yams.constraints import (BASE_CHECKERS, SizeConstraint, RegexpConstraint,
                               StaticVocabularyConstraint, IntervalBoundConstraint,
                               FormatConstraint)
 from yams.reader import SchemaLoader
+from yams.buildobjs import register_base_types
 
 
 # build a dummy schema ########################################################
@@ -362,7 +362,9 @@ class RelationSchemaTC(BaseSchemaTC):
 ##         self.assertRaisesMsg(BadSchemaDefinition,
 ##                              "can't have a final relation pointing to multiple entity types (nom: ['String', 'Int'])" ,
 ##                              rnom.update, enote, eint)
-        self.assertRaisesMsg(BadSchemaDefinition, 'ambiguous relation nom: String is final but not Affaire',
+        msgref = ("ambiguous relation: 'Person.nom' is final (String) "
+                  "but not 'Note.nom' (Affaire)")
+        self.assertRaisesMsg(BadSchemaDefinition, msgref,
                              rnom.update, enote, eaffaire, {})
         self.assertRaises(BadSchemaDefinition,
                           rconcerne.update, enote, estring, {})
@@ -491,6 +493,20 @@ class SchemaTC(BaseSchemaTC):
         self.assertEqual(verr.errors,
                          {None: 'global message about eid 1'})
 
+    def test_validation_error_unicode_then_translation(self):
+        verr = ValidationError(1, {None: 'global message about eid %(eid)s'}, {'eid': 1})
+        self.assertEqual(str(verr), '1 (None): global message about eid 1')
+        self.assertEqual(text_type(verr), '1 (None): global message about eid 1')
+        verr.translate(text_type)
+        self.assertEqual(verr.errors,
+                         {None: 'global message about eid 1'})
+
+    def test_validation_error_translate_without_msgargs(self):
+        """Check that ValidationError.errors get translated even without msgargs"""
+        verr = ValidationError(1, {None: 'hello'})
+        verr.translate(list)
+        self.assertEqual(verr.errors, {None: list('hello')})
+
     def test_pickle(self):
         """schema should be pickeable"""
         import pickle
@@ -534,6 +550,12 @@ class SchemaTC(BaseSchemaTC):
         self.assertListEqual(['custom_workflow'],
                              [rel.name for rel in TE.__relations__])
 
+    def test_add_rdef_after_registration(self):
+        class Label(EntityType):
+            pass
+        Label.expand_type_definitions({})
+        Label.add_relation(RichString(), name='label')
+
 
 class SymetricTC(TestCase):
     def setUp(self):
@@ -590,7 +612,14 @@ class CustomTypeTC(TestCase):
         self.assertIn('Test', RelationDefinitionSchema.BASE_TYPE_PROPERTIES)
         self.assertEqual(RelationDefinitionSchema.BASE_TYPE_PROPERTIES['Test'],
                          {'test1': None, 'test2': None})
-        self.assertTrue('Test' in BASE_CHECKERS)
+        self.assertIn('Test', BASE_CHECKERS)
+        schema = Schema('test')
+        register_base_types(schema)
+        self.assertIn('Test', schema)
+        schema.del_entity_type('Test')
+        self.assertNotIn('Test', BASE_TYPES)
+        self.assertNotIn('Test', RelationDefinitionSchema.BASE_TYPE_PROPERTIES)
+        self.assertNotIn('Test', BASE_CHECKERS)
 
     def test_make_base_type_class(self):
         register_base_type('Test', ('test1', 'test2'))
